import {
  select,
  csv,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  format,
  range,
  map
} from 'd3'; 
import { buttonPanel } from './buttonPanel';

const svg = select('svg.plot')
  .style('background-color', '#edffe3');

const width = +svg.attr('width');
const height = +svg.attr('height');

const margin = { top: 75, right: 60, bottom: 85, left: 135 };
const buttonPanelWidth = 130;
const innerWidth = width - margin.left - margin.right - buttonPanelWidth;
const innerHeight = height - margin.top - margin.bottom;

const features = ['Sepal Length', 'Sepal Width', 'Petal Length', 'Petal Width'];

// d.sepalLength = +d["sepal length"];
// d.sepalWidth = +d["sepal width"];
// d.petalLength = +d["petal length"];
// d.petalWidth = +d["petal width"];
// d.class = +d.class;

const render = (data, selectedX, selectedY) => {
  // console.log(selectedX);
  // console.log(selectedY);
  svg.select('#plotArea').remove();
  const title = 'Scatter Plots of Iris Dataset';
  
  const setosa = data
  	.filter(data => data.class == 'Iris-setosa');
  const versicolor = data
  	.filter(data => data.class == 'Iris-versicolor');
  const virginica = data
  	.filter(data => data.class == 'Iris-virginica');
  
  const setosaColor = '#0f6bff'
  const versicolorColor = '#ff970f'
  const virginicaColor = '#ab0fff'
  // console.log(features[selectedX]);
  // console.log(features[selectedY]);

  const xValue = d => d[features[selectedX]];
  const xAxisLabel = features[selectedX];
  
  const yValue = d => d[features[selectedY]];
  const yAxisLabel = features[selectedY];
  
  const circleRadius = 8;
  
  const xScale = scaleLinear()
  	.domain(extent(data, xValue))
  	.range([0, innerWidth])
  	.nice();
  
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([0, innerHeight])
  	.nice();
  
  const g = svg.append('g')
  	.attr('id', 'plotArea')
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  g.append('rect')
  	.attr('x', 0)
  	.attr('y', 0)
  	.attr('width', innerWidth)
  	.attr('height', innerHeight)
  	.attr('fill', 'white');
  
  const axisTickFormat = number =>
  	format('.2')(number);
  
  const xAxis = axisBottom(xScale)
  	.tickFormat(axisTickFormat)
  	.tickSize(-innerHeight)
  	.tickPadding(10);
  
  const yAxis = axisLeft(yScale)
  	.tickFormat(axisTickFormat)
  	.tickSize(-innerWidth)
  	.tickPadding(10);
  
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform', `translate(0, ${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
  	.attr('class', 'axis-label')
  	.attr('x', innerWidth / 2)
  	.attr('y', 70)
  	.attr('fill', 'black')
  	.text(xAxisLabel);
  
  const yAxisG = g.append('g').call(yAxis)
  	// .attr('transform', `translate(${innerWidth}, 0)`);
	
  yAxisG.select('.domain').remove();
  
  yAxisG.append('text')
  	.attr('class', 'axis-label')
  	.attr('x', -innerHeight / 2)
  	.attr('y', -80)
  	.attr('fill', 'black')
  	.attr('text-anchor', 'middle')
  	.attr('transform', `rotate(-90)`)
  	.text(yAxisLabel);
  
  g.selectAll('scatter').data(setosa)
  	.enter().append('circle')
  		.attr('cx', d => xScale(xValue(d)))
  		.attr('cy', d => yScale(yValue(d)))
  		.attr('r', circleRadius)
  		.attr('fill', setosaColor);
  
  g.selectAll('scatter').data(versicolor)
  	.enter().append('circle')
  		.attr('cx', d => xScale(xValue(d)))
  		.attr('cy', d => yScale(yValue(d)))
  		.attr('r', circleRadius)
  		.attr('fill', versicolorColor);
  
  g.selectAll('scatter').data(virginica)
  	.enter().append('circle')
  		.attr('cx', d => xScale(xValue(d)))
  		.attr('cy', d => yScale(yValue(d)))
  		.attr('r', circleRadius)
  		.attr('fill', virginicaColor);
  
  g.append('text')
  	.attr('class', 'title')
  	// .attr('x', 60)
  	.attr('y', -20)
  	.text(title);
  
  const buttonTexts = g.append('g');
  
  buttonTexts.append('text')
  	.attr('class', 'buttonXY')
  	.attr('x', innerWidth + 10)
  	.attr('y', 30)
  	.text('X axis label:');
  
  buttonTexts.append('text')
  	.attr('class', 'buttonXY')
  	.attr('x', innerWidth + 10)
  	.attr('y', 170)
  	.text('Y axis label:');
  
  buttonTexts.append('rect')
  	.attr('x', innerWidth + 20)
  	.attr('y', 305)
  	.attr('width', buttonPanelWidth + 30)
  	.attr('height', 90)
  	.attr('fill', 'white');
  
  buttonTexts.append('circle')
  	.attr('cx', innerWidth + 35)
    .attr('cy', 320)
    .attr('r', circleRadius)
    .attr('fill', setosaColor);
  
  buttonTexts.append('circle')
  	.attr('cx', innerWidth + 35)
    .attr('cy', 350)
    .attr('r', circleRadius)
    .attr('fill', versicolorColor);
  
  buttonTexts.append('plus')
  	.attr('cx', innerWidth + 35)
    .attr('cy', 380)
    .attr('r', circleRadius)
    .attr('fill', virginicaColor);
  
  buttonTexts.append('text')
  	.attr('class', 'legend')
  	.attr('x', innerWidth + 55)
    .attr('y', 325)
    .text('Iris - setosa');
  
  buttonTexts.append('text')
  	.attr('class', 'legend')
  	.attr('x', innerWidth + 55)
    .attr('y', 355)
    .text('Iris - versicolor');
  
  buttonTexts.append('text')
  	.attr('class', 'legend')
  	.attr('x', innerWidth + 55)
    .attr('y', 385)
    .text('Iris - virginica');
  
};

const makeButton = (type, index) => ({
  type,
  id: index
});

let buttons = range(8)
  .map(d => makeButton('unselected', d));

let selectedX = 0;
let selectedY = 5;

const onClick = (id, data) => {
  if (id < 4) {selectedX = id}
  else {selectedY = id};
  // console.log(selectedX);
  // console.log(selectedY);
  renderButton(data);
};

const renderButton = data => {
  svg.select('#buttonArea').remove();
	const g0 = svg.append('g')
  	.attr('id', 'buttonArea');
  buttonPanel(g0, {
    buttons,
    startX: width - margin.right - buttonPanelWidth,
    startY: margin.top,
    onClick,
    selectedX,
    selectedY,
    data
  });
  render(data, selectedX, selectedY - 4);
};

// csv('http://vis.lab.djosix.com:2020/data/iris.csv')
csv('iris.csv')
  .then(data => {
  	// console.log(data)
    data.filter(d => d.class != '')
      .forEach(d => {
        d['Sepal Length'] = +d["sepal length"];
        d['Sepal Width'] = +d["sepal width"];
        d['Petal Length'] = +d["petal length"];
        d['Petal Width'] = +d["petal width"];
        d.class = d.class;
      });
	renderButton(data);
	});

